@@ -13,9 +13,7 @@
 # General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-# 02111-1307, USA.
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
 
 
 message ()
@@ -490,7 +488,7 @@ case "$FORMAT" in
     else OPTION="-g";
     fi
     ;;
-  txt|rtf|html|xpresstag)
+  txt|rtf|html|xpresstag|mediawiki)
     if [ "$UWORDS" = "no" ]; then OPTION="-n";
     else OPTION="-g";
     fi;
