@@ -12,9 +12,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 #include <apertium/interchunk.h>
 #include <apertium/trx_reader.h>
@@ -33,19 +31,12 @@
 using namespace Apertium;
 using namespace std;
 
-void
-Interchunk::copy(Interchunk const &o)
-{
-}
-
 void
 Interchunk::destroy()
 {
-  if(me)
-  {
-    delete me;
-    me = NULL;
-  }
+  delete me;
+  me = NULL;
+
   if(doc)
   {
     xmlFreeDoc(doc);
@@ -53,7 +44,15 @@ Interchunk::destroy()
   }  
 }
 
-Interchunk::Interchunk()
+Interchunk::Interchunk() :
+word(0),
+blank(0),
+lword(0),
+lblank(0),
+output(0),
+any_char(0),
+any_tag(0),
+nwords(0)
 {
   me = NULL;
   doc = NULL;
@@ -71,22 +70,6 @@ Interchunk::~Interchunk()
   destroy();
 }
 
-Interchunk::Interchunk(Interchunk const &o)
-{
-  copy(o);
-}
-
-Interchunk &
-Interchunk::operator =(Interchunk const &o)
-{
-  if(this != &o)
-  {
-    destroy();
-    copy(o);
-  }
-  return *this;
-}
-
 void 
 Interchunk::readData(FILE *in)
 {
@@ -241,6 +224,11 @@ Interchunk::checkIndex(xmlNode *element, int index, int limit)
 string 
 Interchunk::evalString(xmlNode *element)
 {
+  if (element == 0)
+  {
+    throw "Interchunk::evalString() was passed a NULL element";
+  }
+
   map<xmlNode *, TransferInstr>::iterator it;
   it = evalStringCache.find(element);
   if(it != evalStringCache.end())
@@ -630,6 +618,8 @@ Interchunk::processCallMacro(xmlNode *localroot)
     }
   }
 
+  // ToDo: Is it at all valid if npar <= 0 ?
+
   InterchunkWord **myword = NULL;
   if(npar > 0)
   {
@@ -644,7 +634,7 @@ Interchunk::processCallMacro(xmlNode *localroot)
 
   int idx = 0;
   int lastpos = 0;
-  for(xmlNode *i = localroot->children; i != NULL; i = i->next)
+  for(xmlNode *i = localroot->children; npar && i != NULL; i = i->next)
   {
     if(i->type == XML_ELEMENT_NODE)
     {
@@ -675,14 +665,8 @@ Interchunk::processCallMacro(xmlNode *localroot)
   swap(myblank, blank);
   swap(npar, lword);
 
-  if(myword)
-  {
-    delete[] myword;
-  }
-  if(myblank)
-  {
-    delete[] myblank;
-  }
+  delete[] myword;
+  delete[] myblank;
 }
 
 void
