@@ -13,9 +13,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <apertium/lextor.h>
@@ -34,12 +32,16 @@ using namespace Apertium;
 bool LexTor::debug;
 double LexTor::angleth;
 
-LexTor::LexTor() {
+LexTor::LexTor() :
+fstpbil(0)
+{
   lextor_data=NULL;
   tlmodel=NULL;
 }
   
-LexTor::LexTor(const LexTor& lt) {
+LexTor::LexTor(const LexTor& lt) :
+fstpbil(0)
+{
   lextor_data=lt.lextor_data;
   tlmodel=lt.tlmodel;
 }
@@ -335,11 +337,7 @@ LexTor::trainlch(wistream& is, int left, int right, LexTorData& tlwordmodel,
 		  //The counts of the TL co-occurrence model are transferred to the SL. If the SL word is ambiguous
 		  //it will have more than one translation into TL, so we need to normalize using the frequency of words
 		  //in the TL
-#ifdef __GNUC__
-		  double translation_weighs[translation_buffer[i].size()];
-#else
 		  vector <double> translation_weighs(translation_buffer[i].size());
-#endif
 		  double sum=0.0;
 		  if (translation_buffer[i].size()>1) {
 		    for(int j=0; j<(int)translation_buffer[i].size(); j++) {
@@ -681,11 +679,7 @@ LexTor::estimate_winner_lch(deque<LexTorWord>& window, int word_index, double we
 
 int 
 LexTor::estimate_winner_lch_voting(deque<LexTorWord>& window, int word_index, double weigth_exponent) {
-#ifdef __GNUC__
-  double lexchoices_count[window[word_index].n_lexical_choices()];
-#else
   vector <double> lexchoices_count(window[word_index].n_lexical_choices());
-#endif
 
   if (debug) {
     wcerr<<L"WINDOW: ";
@@ -924,11 +918,7 @@ LexTor::estimate_winner_lch_votingtl(deque<LexTorWord>& window, int word_index,
 	//If the SL word is ambiguous it will have more than one
 	//translation into TL, so we need to normalize using the
 	//frequency of words in the TL
-#ifdef __GNUC__
-	double translation_weighs[translation_window[k].size()];
-#else
     vector <double> translation_weighs(translation_window[k].size());
-#endif
 	double sum=0.0;
 	if (translation_window[k].size()>1) {
 	  for(unsigned j=0; j<translation_window[k].size(); j++) {
