@@ -12,9 +12,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 #ifndef _POSTCHUNK_
 #define _POSTCHUNK_
@@ -74,7 +72,6 @@ private:
   bool null_flush;
   bool internal_null_flush;
 
-  void copy(Postchunk const &o);
   void destroy();
   void readData(FILE *input);
   void readPostchunk(string const &input);
@@ -131,14 +128,11 @@ private:
 public:
   Postchunk();
   ~Postchunk();
-  Postchunk(Postchunk const &o);
-  Postchunk & operator =(Postchunk const &o);
   
   void read(string const &transferfile, string const &datafile);
   void postchunk(FILE *in, FILE *out);
   bool getNullFlush(void);
   void setNullFlush(bool null_flush);
-
 };
 
 #endif
