@@ -12,9 +12,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 #include <apertium/transfer.h>
 #include <apertium/trx_reader.h>
@@ -32,11 +30,6 @@
 using namespace Apertium;
 using namespace std;
 
-void
-Transfer::copy(Transfer const &o)
-{
-}
-
 void
 Transfer::destroy()
 {
@@ -49,10 +42,18 @@ Transfer::destroy()
   {
     xmlFreeDoc(doc);
     doc = NULL;
-  }  
+  }
 }
 
-Transfer::Transfer()
+Transfer::Transfer() :
+word(0),
+blank(0),
+lword(0),
+lblank(0),
+output(0),
+any_char(0),
+any_tag(0),
+nwords(0)
 {
   me = NULL;
   doc = NULL;
@@ -65,6 +66,7 @@ Transfer::Transfer()
   null_flush = false;
   internal_null_flush = false;
   trace = false;
+  trace_att = false;
   emptyblank = "";
 }
 
@@ -73,23 +75,7 @@ Transfer::~Transfer()
   destroy();
 }
 
-Transfer::Transfer(Transfer const &o)
-{
-  copy(o);
-}
-
-Transfer &
-Transfer::operator =(Transfer const &o)
-{
-  if(this != &o)
-  {
-    destroy();
-    copy(o);
-  }
-  return *this;
-}
-
-void 
+void
 Transfer::readData(FILE *in)
 {
   alphabet.read(in);
@@ -98,18 +84,18 @@ Transfer::readData(FILE *in)
 
   Transducer t;
   t.read(in, alphabet.size());
-  
-  map<int, int> finals;  
-  
+
+  map<int, int> finals;
+
   // finals
   for(int i = 0, limit = Compression::multibyte_read(in); i != limit; i++)
   {
     int key = Compression::multibyte_read(in);
     finals[key] = Compression::multibyte_read(in);
-  }  
-  
+  }
+
   me = new MatchExe(t, finals);
- 
+
   // attr_items
   bool recompile_attrs = Compression::string_read(in) != string(pcre_version());
   for(int i = 0, limit = Compression::multibyte_read(in); i != limit; i++)
@@ -146,7 +132,7 @@ Transfer::readData(FILE *in)
       wstring const cad_v = Compression::wstring_read(in);
       lists[cad_k].insert(UtfConverter::toUtf8(cad_v));
       listslow[cad_k].insert(UtfConverter::toUtf8(StringUtils::tolower(cad_v)));
-    }  
+    }
   }
 }
 
@@ -184,7 +170,7 @@ Transfer::read(string const &transferfile, string const &datafile,
 	       string const &fstfile)
 {
   readTransfer(transferfile);
-  
+
   // datafile
   FILE *in = fopen(datafile.c_str(), "rb");
   if(!in)
@@ -194,7 +180,7 @@ Transfer::read(string const &transferfile, string const &datafile,
   }
   readData(in);
   fclose(in);
-  
+
   if(fstfile != "")
   {
     readBil(fstfile);
@@ -205,15 +191,15 @@ void
 Transfer::readTransfer(string const &in)
 {
   doc = xmlReadFile(in.c_str(), NULL, 0);
-  
+
   if(doc == NULL)
   {
     cerr << "Error: Could not parse file '" << in << "'." << endl;
     exit(EXIT_FAILURE);
   }
-  
+
   root_element = xmlDocGetRootElement(doc);
-  
+
   // search for root element attributes
   for(xmlAttr *i = root_element->properties; i != NULL; i = i->next)
   {
@@ -227,9 +213,9 @@ Transfer::readTransfer(string const &in)
       {
         defaultAttrs = lu; // default value for 'default'
       }
-    }  
+    }
   }
-  
+
   // search for macros & rules
   for(xmlNode *i = root_element->children; i != NULL; i = i->next)
   {
@@ -243,7 +229,7 @@ Transfer::readTransfer(string const &in)
       {
         collectRules(i);
       }
-    } 
+    }
   }
 }
 
@@ -268,11 +254,11 @@ Transfer::collectRules(xmlNode *localroot)
 
 void
 Transfer::collectMacros(xmlNode *localroot)
-{ 
+{
   for(xmlNode *i = localroot->children; i != NULL; i = i->next)
   {
     if(i->type == XML_ELEMENT_NODE)
-    {     
+    {
       macro_map.push_back(i);
     }
   }
@@ -290,7 +276,7 @@ Transfer::checkIndex(xmlNode *element, int index, int limit)
 }
 
 
-string 
+string
 Transfer::evalString(xmlNode *element)
 {
   map<xmlNode *, TransferInstr>::iterator it;
@@ -313,42 +299,42 @@ Transfer::evalString(xmlNode *element)
           return word[ti.getPos()]->target(attr_items[ti.getContent()], ti.getCondition());
         }
         break;
-      
+
       case ti_linkto_sl:
         if(checkIndex(element, ti.getPos(), lword))
         {
           if(word[ti.getPos()]->source(attr_items[ti.getContent()], ti.getCondition()) != "")
           {
             return "<" + string((char *) ti.getPointer()) + ">";
-          }        
+          }
           else
           {
             return "";
           }
         }
         break;
-        
+
       case ti_linkto_tl:
         if(checkIndex(element, ti.getPos(), lword))
         {
           if(word[ti.getPos()]->target(attr_items[ti.getContent()], ti.getCondition()) != "")
           {
             return "<" + string((char *) ti.getPointer()) + ">";
-          }        
+          }
           else
           {
             return "";
           }
         }
         break;
-          
+
       case ti_var:
         return variables[ti.getContent()];
 
       case ti_lit_tag:
       case ti_lit:
         return ti.getContent();
-        
+
       case ti_b:
         if(checkIndex(element, ti.getPos(), lblank))
         {
@@ -359,7 +345,7 @@ Transfer::evalString(xmlNode *element)
           return " ";
         }
         break;
-        
+
       case ti_get_case_from:
         if(checkIndex(element, ti.getPos(), lword))
         {
@@ -367,21 +353,21 @@ Transfer::evalString(xmlNode *element)
                   evalString((xmlNode *) ti.getPointer()));
         }
         break;
-        
+
       case ti_case_of_sl:
         if(checkIndex(element, ti.getPos(), lword))
         {
           return caseOf(word[ti.getPos()]->source(attr_items[ti.getContent()]));
         }
         break;
-      
+
       case ti_case_of_tl:
         if(checkIndex(element, ti.getPos(), lword))
         {
           return caseOf(word[ti.getPos()]->target(attr_items[ti.getContent()]));
         }
         break;
-        
+
       default:
         return "";
     }
@@ -430,8 +416,8 @@ Transfer::evalString(xmlNode *element)
       else
       {
         evalStringCache[element] = TransferInstr(ti_linkto_tl, (const char *) part, pos, (void *) as, queue);
-      }      
-    }      
+      }
+    }
     else if(!xmlStrcmp(side, (const xmlChar *) "sl"))
     {
       evalStringCache[element] = TransferInstr(ti_clip_sl, (const char *) part, pos, NULL, queue);
@@ -443,8 +429,8 @@ Transfer::evalString(xmlNode *element)
   }
   else if(!xmlStrcmp(element->name, (const xmlChar *) "lit-tag"))
   {
-    evalStringCache[element] = TransferInstr(ti_lit_tag, 
-                                             tags((const char *) element->properties->children->content), 0);                                            
+    evalStringCache[element] = TransferInstr(ti_lit_tag,
+                                             tags((const char *) element->properties->children->content), 0);
   }
   else if(!xmlStrcmp(element->name, (const xmlChar *) "lit"))
   {
@@ -501,7 +487,7 @@ Transfer::evalString(xmlNode *element)
 	pos = atoi((const char *) i->children->content) - 1;
       }
     }
-      
+
     if(!xmlStrcmp(side, (const xmlChar *) "sl"))
     {
       evalStringCache[element] = TransferInstr(ti_case_of_sl, (const char *) part, pos);
@@ -509,10 +495,10 @@ Transfer::evalString(xmlNode *element)
     else
     {
       evalStringCache[element] = TransferInstr(ti_case_of_tl, (const char *) part, pos);
-    }    
+    }
   }
   else if(!xmlStrcmp(element->name, (const xmlChar *) "concat"))
-  { 
+  {
     string value;
     for(xmlNode *i = element->children; i != NULL; i = i->next)
     {
@@ -533,7 +519,7 @@ Transfer::evalString(xmlNode *element)
          myword.append(evalString(i));
        }
     }
-    
+
     if(myword != "")
     {
       return "^"+myword+"$";
@@ -546,15 +532,15 @@ Transfer::evalString(xmlNode *element)
   else if(!xmlStrcmp(element->name, (const xmlChar *) "mlu"))
   {
     string value;
-     	  
+
     bool first_time = true;
-    
+
     for(xmlNode *i = element->children; i != NULL; i = i->next)
     {
       if(i->type == XML_ELEMENT_NODE)
       {
         string myword;
-	 
+
         for(xmlNode *j = i->children; j != NULL; j = j->next)
         {
           if(j->type == XML_ELEMENT_NODE)
@@ -562,7 +548,7 @@ Transfer::evalString(xmlNode *element)
             myword.append(evalString(j));
 	  }
         }
-	      
+
 	if(!first_time)
 	{
 	  if(myword != "" && myword[0] != '#')  //'+#' problem
@@ -577,7 +563,7 @@ Transfer::evalString(xmlNode *element)
 	    first_time = false;
           }
 	}
-	 
+
 	value.append(myword);
       }
     }
@@ -646,7 +632,7 @@ Transfer::processOut(xmlNode *localroot)
                   myword.append(evalString(k));
 	        }
 	      }
-	      
+
 	      if(!first_time)
 	      {
 	        if(myword != "" && myword[0] != '#')  //'+#' problem
@@ -668,11 +654,11 @@ Transfer::processOut(xmlNode *localroot)
         }
         else // 'b'
         {
-          fputws_unlocked(UtfConverter::fromUtf8(evalString(i)).c_str(), 
+          fputws_unlocked(UtfConverter::fromUtf8(evalString(i)).c_str(),
 			  output);
         }
       }
-      else 
+      else
       {
         if(!xmlStrcmp(i->name, (const xmlChar *) "chunk"))
         {
@@ -681,7 +667,7 @@ Transfer::processOut(xmlNode *localroot)
         else // 'b'
         {
           fputws_unlocked(UtfConverter::fromUtf8(evalString(i)).c_str(), output);
-        }        
+        }
       }
     }
   }
@@ -693,8 +679,8 @@ Transfer::processChunk(xmlNode *localroot)
   string name, namefrom;
   string caseofchunk = "aa";
   string result;
-      
-  
+
+
   for(xmlAttr *i = localroot->properties; i != NULL; i = i->next)
   {
     if(!xmlStrcmp(i->name, (const xmlChar *) "name"))
@@ -744,7 +730,7 @@ Transfer::processChunk(xmlNode *localroot)
       exit(EXIT_FAILURE);
     }
   }
-  
+
   for(xmlNode *i = localroot->children; i != NULL; i = i->next)
   {
     if(i->type == XML_ELEMENT_NODE)
@@ -765,7 +751,7 @@ Transfer::processChunk(xmlNode *localroot)
           }
         }
         if(myword != "")
-        { 
+        {
           result.append("^");
           result.append(myword);
           result.append("$");
@@ -787,12 +773,12 @@ Transfer::processChunk(xmlNode *localroot)
                 mylocalword.append(evalString(k));
               }
             }
-          
+
             if(!first_time)
             {
               if(mylocalword != "" && mylocalword[0] != '#')  // '+#' problem
               {
-                myword += '+';	
+                myword += '+';
               }
             }
             else
@@ -842,12 +828,13 @@ Transfer::processTags(xmlNode *localroot)
   return result;
 }
 
-void
+int
 Transfer::processInstruction(xmlNode *localroot)
 {
+  int words_to_consume = -1;
   if(!xmlStrcmp(localroot->name, (const xmlChar *) "choose"))
   {
-    processChoose(localroot);
+    words_to_consume = processChoose(localroot);
   }
   else if(!xmlStrcmp(localroot->name, (const xmlChar *) "let"))
   {
@@ -869,6 +856,33 @@ Transfer::processInstruction(xmlNode *localroot)
   {
     processModifyCase(localroot);
   }
+  else if(!xmlStrcmp(localroot->name, (const xmlChar *) "reject-current-rule"))
+  {
+    words_to_consume = processRejectCurrentRule(localroot);
+  }
+  return words_to_consume;
+}
+
+int
+Transfer::processRejectCurrentRule(xmlNode *localroot)
+{
+  bool shifting = true;
+  string value;
+  for(xmlAttr *i = localroot->properties; i != NULL; i = i->next)
+  {
+    if(!xmlStrcmp(i->name, (const xmlChar *) "shifting"))
+    {
+      value = (char *) i->children->content;
+      break;
+    }
+  }
+
+  if(value == "no")
+  {
+    shifting = false;
+  }
+
+  return shifting ? 1 : 0;
 }
 
 void
@@ -901,15 +915,15 @@ Transfer::processLet(xmlNode *localroot)
       case ti_var:
         variables[ti.getContent()] = evalString(rightSide);
         return;
-        
+
       case ti_clip_sl:
         word[ti.getPos()]->setSource(attr_items[ti.getContent()], evalString(rightSide), ti.getCondition());
         return;
-      
+
       case ti_clip_tl:
         word[ti.getPos()]->setTarget(attr_items[ti.getContent()], evalString(rightSide), ti.getCondition());
-        return;      
-        
+        return;
+
       default:
         return;
     }
@@ -950,9 +964,9 @@ Transfer::processLet(xmlNode *localroot)
       else if(!xmlStrcmp(i->name, (const xmlChar *) "link-to"))
       {
         as = i->children->content;
-      }      
+      }
     }
-    
+
     if(!xmlStrcmp(side, (const xmlChar *) "tl"))
     {
       word[pos]->setTarget(attr_items[(const char *) part], evalString(rightSide), queue);
@@ -962,7 +976,7 @@ Transfer::processLet(xmlNode *localroot)
     {
       word[pos]->setSource(attr_items[(const char *) part], evalString(rightSide), queue);
       evalStringCache[leftSide] = TransferInstr(ti_clip_sl, (const char *) part, pos, NULL, queue);
-    }    
+    }
   }
 }
 
@@ -974,7 +988,7 @@ Transfer::processAppend(xmlNode *localroot)
   {
     if(!xmlStrcmp(i->name, (const xmlChar *) "n"))
     {
-      name = (char *) i->children->content; 
+      name = (char *) i->children->content;
       break;
     }
   }
@@ -1035,21 +1049,22 @@ Transfer::processModifyCase(xmlNode *localroot)
         {
           queue = false;
         }
-      }  
+      }
       else if(!xmlStrcmp(i->name, (const xmlChar *) "link-to"))
       {
         as = i->children->content;
+        (void)as; // ToDo, remove "as" and the whole else?
       }
     }
     if(!xmlStrcmp(side, (const xmlChar *) "sl"))
     {
-      string const result = copycase(evalString(rightSide), 
+      string const result = copycase(evalString(rightSide),
 				      word[pos]->source(attr_items[(const char *) part], queue));
       word[pos]->setSource(attr_items[(const char *) part], result);
     }
     else
     {
-      string const result = copycase(evalString(rightSide), 
+      string const result = copycase(evalString(rightSide),
 				     word[pos]->target(attr_items[(const char *) part], queue));
       word[pos]->setTarget(attr_items[(const char *) part], result);
     }
@@ -1077,11 +1092,13 @@ Transfer::processCallMacro(xmlNode *localroot)
       break;
     }
   }
+  
+  // ToDo: Is it at all valid if npar <= 0 ?
 
   TransferWord **myword = NULL;
   if(npar > 0)
   {
-    myword = new TransferWord *[npar];  
+    myword = new TransferWord *[npar];
   }
   string **myblank = NULL;
   if(npar > 0)
@@ -1092,7 +1109,7 @@ Transfer::processCallMacro(xmlNode *localroot)
 
   int idx = 0;
   int lastpos = 0;
-  for(xmlNode *i = localroot->children; i != NULL; i = i->next)
+  for(xmlNode *i = localroot->children; npar && i != NULL; i = i->next)
   {
     if(i->type == XML_ELEMENT_NODE)
     {
@@ -1110,7 +1127,7 @@ Transfer::processCallMacro(xmlNode *localroot)
   swap(myword, word);
   swap(myblank, blank);
   swap(npar, lword);
-  
+
   for(xmlNode *i = macro->children; i != NULL; i = i->next)
   {
     if(i->type == XML_ELEMENT_NODE)
@@ -1123,19 +1140,14 @@ Transfer::processCallMacro(xmlNode *localroot)
   swap(myblank, blank);
   swap(npar, lword);
   
-  if(myword)
-  {
-    delete[] myword;
-  }
-  if(myblank)
-  {
-    delete[] myblank;
-  }
+  delete[] myword;
+  delete[] myblank;
 }
 
-void
+int
 Transfer::processChoose(xmlNode *localroot)
 {
+  int words_to_consume = -1;
   for(xmlNode *i = localroot->children; i != NULL; i = i->next)
   {
     if(i->type == XML_ELEMENT_NODE)
@@ -1143,7 +1155,7 @@ Transfer::processChoose(xmlNode *localroot)
       if(!xmlStrcmp(i->name, (const xmlChar *) "when"))
       {
         bool picked_option = false;
-        
+
 	for(xmlNode *j = i->children; j != NULL; j = j->next)
 	{
 	  if(j->type == XML_ELEMENT_NODE)
@@ -1161,14 +1173,18 @@ Transfer::processChoose(xmlNode *localroot)
 	    }
 	    else
 	    {
-	      processInstruction(j);
+              words_to_consume = processInstruction(j);
+              if(words_to_consume != -1)
+              {
+                return words_to_consume;
+              }
 	    }
 	  }
 	}
         if(picked_option)
         {
-          return;
-        }	
+          return words_to_consume;
+        }
       }
       else if(!xmlStrcmp(i->name, (const xmlChar *) "otherwise"))
       {
@@ -1176,12 +1192,17 @@ Transfer::processChoose(xmlNode *localroot)
 	{
 	  if(j->type == XML_ELEMENT_NODE)
 	  {
-	    processInstruction(j);
-	  }
-	}
+            words_to_consume = processInstruction(j);
+            if(words_to_consume != -1)
+            {
+              return words_to_consume;
+            }
+          }
+        }
       }
     }
   }
+  return words_to_consume;
 }
 
 bool
@@ -1222,7 +1243,7 @@ Transfer::processLogical(xmlNode *localroot)
   else if(!xmlStrcmp(localroot->name, (const xmlChar *) "not"))
   {
     return processNot(localroot);
-  } 
+  }
   else if(!xmlStrcmp(localroot->name, (const xmlChar *) "in"))
   {
     return processIn(localroot);
@@ -1240,15 +1261,15 @@ Transfer::processIn(xmlNode *localroot)
   for(xmlNode *i = localroot->children; i != NULL; i = i->next)
   {
     if(i->type == XML_ELEMENT_NODE)
-    { 
+    {
       if(value == NULL)
       {
-	value = i;
+        value = i;
       }
       else
       {
-	idlist = i->properties->children->content;
-	break;
+        idlist = i->properties->children->content;
+        break;
       }
     }
   }
@@ -1257,7 +1278,7 @@ Transfer::processIn(xmlNode *localroot)
 
   if(localroot->properties != NULL)
   {
-    if(!xmlStrcmp(localroot->properties->children->content, 
+    if(!xmlStrcmp(localroot->properties->children->content,
 		  (const xmlChar *) "yes"))
     {
       set<string, Ltstr> &myset = listslow[(const char *) idlist];
@@ -1292,7 +1313,7 @@ Transfer::processTest(xmlNode *localroot)
     {
       return processLogical(i);
     }
-  }  
+  }
   return false;
 }
 
@@ -1372,7 +1393,7 @@ Transfer::processEqual(xmlNode *localroot)
       return tolower(evalString(first)) == tolower(evalString(second));
     }
     else
-    { 
+    {
       return evalString(first) == evalString(second);
     }
   }
@@ -1382,7 +1403,7 @@ bool
 Transfer::beginsWith(string const &s1, string const &s2) const
 {
   int const limit = s2.size(), constraint = s1.size();
-  
+
   if(constraint < limit)
   {
     return false;
@@ -1402,7 +1423,7 @@ bool
 Transfer::endsWith(string const &s1, string const &s2) const
 {
   int const limit = s2.size(), constraint = s1.size();
-  
+
   if(constraint < limit)
   {
     return false;
@@ -1522,7 +1543,7 @@ Transfer::processBeginsWithList(xmlNode *localroot)
   string needle = evalString(first);
   set<string, Ltstr>::iterator it, limit;
 
-  if(localroot->properties == NULL || 
+  if(localroot->properties == NULL ||
      xmlStrcmp(localroot->properties->children->content, (const xmlChar *) "yes"))
   {
     it = lists[(const char *) idlist].begin();
@@ -1534,7 +1555,7 @@ Transfer::processBeginsWithList(xmlNode *localroot)
     it = listslow[(const char *) idlist].begin();
     limit = listslow[(const char *) idlist].end();
   }
-  
+
   for(; it != limit; it++)
   {
     if(beginsWith(needle, *it))
@@ -1571,7 +1592,7 @@ Transfer::processEndsWithList(xmlNode *localroot)
   string needle = evalString(first);
   set<string, Ltstr>::iterator it, limit;
 
-  if(localroot->properties == NULL || 
+  if(localroot->properties == NULL ||
      xmlStrcmp(localroot->properties->children->content, (const xmlChar *) "yes"))
   {
     it = lists[(const char *) idlist].begin();
@@ -1583,7 +1604,7 @@ Transfer::processEndsWithList(xmlNode *localroot)
     it = listslow[(const char *) idlist].begin();
     limit = listslow[(const char *) idlist].end();
   }
-  
+
   for(; it != limit; it++)
   {
     if(endsWith(needle, *it))
@@ -1654,16 +1675,16 @@ Transfer::copycase(string const &source_word, string const &target_word)
   {
     result = StringUtils::toupper(t_word);
   }
-  
+
   if(firstupper)
   {
     result[0] = towupper(result[0]);
   }
-   
+
   return UtfConverter::toUtf8(result);
 }
 
-string 
+string
 Transfer::caseOf(string const &str)
 {
   wstring const s = UtfConverter::fromUtf8(str);
@@ -1722,23 +1743,32 @@ Transfer::tags(string const &str) const
       result += str[i];
     }
   }
-  
+
   result += '>';
 
   return result;
 }
 
-void
+int
 Transfer::processRule(xmlNode *localroot)
 {
+  int instruction_return, words_to_consume = -1;
   // localroot is suposed to be an 'action' tag
   for(xmlNode *i = localroot->children; i != NULL; i = i->next)
   {
     if(i->type == XML_ELEMENT_NODE)
     {
-      processInstruction(i);
+      instruction_return = processInstruction(i);
+      // When an instruction which modifies the number of words to be consumed
+      // from the input is found, execution of the rule is stopped
+      if(instruction_return != -1)
+      {
+        words_to_consume = instruction_return;
+        break;
+      }
     }
   }
+  return words_to_consume;
 }
 
 TransferToken &
@@ -1758,7 +1788,7 @@ Transfer::readToken(FILE *in)
       return input_buffer.add(TransferToken(content, tt_eof));
     }
     if(val == '\\')
-    {  
+    {
       content += L'\\';
       content += (wchar_t) fgetwc_unlocked(in);
     }
@@ -1821,12 +1851,18 @@ Transfer::setTrace(bool trace)
   this->trace = trace;
 }
 
+void
+Transfer::setTraceATT(bool trace)
+{
+  this->trace_att = trace;
+}
+
 void
 Transfer::transfer_wrapper_null_flush(FILE *in, FILE *out)
 {
   null_flush = false;
   internal_null_flush = true;
-  
+
   while(!feof(in))
   {
     transfer(in, out);
@@ -1840,7 +1876,7 @@ Transfer::transfer_wrapper_null_flush(FILE *in, FILE *out)
 
   internal_null_flush = false;
   null_flush = true;
-}    
+}
 
 void
 Transfer::transfer(FILE *in, FILE *out)
@@ -1849,28 +1885,107 @@ Transfer::transfer(FILE *in, FILE *out)
   {
     transfer_wrapper_null_flush(in, out);
   }
-  
+
   int last = 0;
+  int prev_last = 0;
+  int lastrule_id = -1;
+  set<int> banned_rules;
 
   output = out;
   ms.init(me->getInitial());
-  
+
   while(true)
   {
+    if(trace_att)
+    {
+      cerr << "Loop start " << endl;
+      cerr << "ms.size: " << ms.size() << endl;
+
+      cerr << "tmpword.size(): " << tmpword.size() << endl;
+      for (unsigned int ind = 0; ind < tmpword.size(); ind++)
+      {
+        if(ind != 0)
+        {
+          wcerr << L" ";
+        }
+        wcerr << *tmpword[ind];
+      }
+      wcerr << endl;
+
+      cerr << "tmpblank.size(): " << tmpblank.size() << endl;
+      for (unsigned int ind = 0; ind < tmpblank.size(); ind++)
+      {
+        wcerr << L"'";
+        wcerr << *tmpblank[ind];
+        wcerr << L"' ";
+      }
+      wcerr << endl;
+
+      cerr << "last: " << last << endl;
+      cerr << "prev_last: " << prev_last << endl << endl;
+    }
+
     if(ms.size() == 0)
     {
       if(lastrule != NULL)
       {
-	applyRule();
-	input_buffer.setPos(last);
+        int num_words_to_consume = applyRule();
+
+        if(trace_att)
+        {
+          cerr << "num_words_to_consume: " << num_words_to_consume << endl;
+        }
+
+        //Consume all the words from the input which matched the rule.
+        //This piece of code is executed unless the rule contains a "reject-current-rule" instruction
+        if(num_words_to_consume < 0)
+        {
+          banned_rules.clear();
+          input_buffer.setPos(last);
+        }
+        else if(num_words_to_consume > 0)
+        {
+          banned_rules.clear();
+          if(prev_last >= input_buffer.getSize())
+          {
+            input_buffer.setPos(0);
+          }
+          else
+          {
+            input_buffer.setPos(prev_last+1);
+          }
+          int num_consumed_words = 0;
+          while(num_consumed_words < num_words_to_consume)
+          {
+            TransferToken& local_tt = input_buffer.next();
+            if (local_tt.getType() == tt_word)
+            {
+              num_consumed_words++;
+            }
+          }
+        }
+        else
+        {
+          //Add rule to banned rules
+          banned_rules.insert(lastrule_id);
+          input_buffer.setPos(prev_last);
+          input_buffer.next();
+          last = input_buffer.getPos();
+        }
+        lastrule_id = -1;
       }
       else
       {
-	if(tmpword.size() != 0)
-	{
-	  pair<wstring, int> tr;
-	  if(useBilingual && preBilingual == false)
-	  {
+        if(tmpword.size() != 0)
+        {
+          if(trace_att)
+          {
+            cerr << "printing tmpword[0]" <<endl;
+          }
+
+          pair<wstring, int> tr;
+          if(useBilingual && preBilingual == false)
+          {
 	    if(isExtended && (*tmpword[0])[0] == L'*')
 	    {
 	      tr = extended.biltransWithQueue((*tmpword[0]).substr(1), false);
@@ -1893,7 +2008,7 @@ Transfer::transfer(FILE *in, FILE *out)
             wstring sl;
             wstring tl;
             int seenSlash = 0;
-            for(wstring::const_iterator it = tmpword[0]->begin(); it != tmpword[0]->end(); it++) 
+            for(wstring::const_iterator it = tmpword[0]->begin(); it != tmpword[0]->end(); it++)
             {
               if(*it == L'\\')
               {
@@ -1911,7 +2026,7 @@ Transfer::transfer(FILE *in, FILE *out)
                 }
                 continue;
               }
-              else if(*it == L'/') 
+              else if(*it == L'/')
               {
                 seenSlash++;
                 continue;
@@ -1929,15 +2044,15 @@ Transfer::transfer(FILE *in, FILE *out)
                 break;
               }
             }
-            //tmpword[0]->assign(sl); 
+            //tmpword[0]->assign(sl);
             tr = pair<wstring, int>(tl, false);
-            //wcerr << L"pb: " << *tmpword[0] << L" :: " << sl << L" >> " << tl << endl ; 
+            //wcerr << L"pb: " << *tmpword[0] << L" :: " << sl << L" >> " << tl << endl ;
           }
           else
           {
             tr = pair<wstring, int>(*tmpword[0], 0);
           }
-          
+
 	  if(tr.first.size() != 0)
 	  {
 	    if(defaultAttrs == lu)
@@ -1953,32 +2068,40 @@ Transfer::transfer(FILE *in, FILE *out)
                 fputws_unlocked(L"^unknown<unknown>{^", output);
               }
               else
-              {                
+              {
 	        fputws_unlocked(L"^default<default>{^", output);
-              }	        
+              }
 	      fputws_unlocked(tr.first.c_str(), output);
 	      fputws_unlocked(L"$}$", output);
             }
 	  }
+	  banned_rules.clear();
 	  tmpword.clear();
 	  input_buffer.setPos(last);
-	  input_buffer.next();       
+	  input_buffer.next();
+	  prev_last = last;
 	  last = input_buffer.getPos();
 	  ms.init(me->getInitial());
 	}
 	else if(tmpblank.size() != 0)
 	{
-	  fputws_unlocked(tmpblank[0]->c_str(), output);
-	  tmpblank.clear();
-	  last = input_buffer.getPos();
-	  ms.init(me->getInitial());
+          if(trace_att)
+          {
+            cerr << "printing tmpblank[0]" <<endl;
+          }
+          fputws_unlocked(tmpblank[0]->c_str(), output);
+          tmpblank.clear();
+          prev_last = last;
+          last = input_buffer.getPos();
+          ms.init(me->getInitial());
 	}
       }
     }
-    int val = ms.classifyFinals(me->getFinals());
+    int val = ms.classifyFinals(me->getFinals(), banned_rules);
     if(val != -1)
     {
-      lastrule = rule_map[val-1];      
+      lastrule = rule_map[val-1];
+      lastrule_id = val;
       last = input_buffer.getPos();
 
       if(trace)
@@ -1997,7 +2120,7 @@ Transfer::transfer(FILE *in, FILE *out)
     }
 
     TransferToken &current = readToken(in);
-   
+
     switch(current.getType())
     {
       case tt_word:
@@ -2030,12 +2153,13 @@ Transfer::transfer(FILE *in, FILE *out)
   }
 }
 
-void
+int
 Transfer::applyRule()
 {
+  int words_to_consume;
   unsigned int limit = tmpword.size();
   //wcerr << L"applyRule: " << tmpword.size() << endl;
-  
+
   for(unsigned int i = 0; i != limit; i++)
   {
     if(i == 0)
@@ -2057,7 +2181,7 @@ Transfer::applyRule()
     {
       blank[i-1] = new string(UtfConverter::toUtf8(*tmpblank[i-1]));
     }
-    
+
     pair<wstring, int> tr;
     if(useBilingual && preBilingual == false)
     {
@@ -2069,7 +2193,7 @@ Transfer::applyRule()
       wstring sl;
       wstring tl;
       int seenSlash = 0;
-      for(wstring::const_iterator it = tmpword[i]->begin(); it != tmpword[i]->end(); it++) 
+      for(wstring::const_iterator it = tmpword[i]->begin(); it != tmpword[i]->end(); it++)
       {
         if(*it == L'\\')
         {
@@ -2088,7 +2212,7 @@ Transfer::applyRule()
           continue;
         }
 
-        if(*it == L'/') 
+        if(*it == L'/')
         {
           seenSlash++;
           continue;
@@ -2105,8 +2229,8 @@ Transfer::applyRule()
         {
           break;
         }
-      } 
-      //tmpword[i]->assign(sl); 
+      }
+      //tmpword[i]->assign(sl);
       tr = pair<wstring, int>(tl, false);
     }
     else
@@ -2114,11 +2238,11 @@ Transfer::applyRule()
       tr = pair<wstring, int>(*tmpword[i], false);
     }
 
-    word[i] = new TransferWord(UtfConverter::toUtf8(*tmpword[i]), 
+    word[i] = new TransferWord(UtfConverter::toUtf8(*tmpword[i]),
 			       UtfConverter::toUtf8(tr.first), tr.second);
   }
 
-  processRule(lastrule);
+  words_to_consume = processRule(lastrule);
   lastrule = NULL;
 
   if(word)
@@ -2142,6 +2266,7 @@ Transfer::applyRule()
   tmpword.clear();
   tmpblank.clear();
   ms.init(me->getInitial());
+  return words_to_consume;
 }
 
 /* HERE */
@@ -2181,7 +2306,7 @@ Transfer::applyWord(wstring const &word_str)
 	  }
 	}
 	break;
-	
+
       default:
 	ms.step(towlower(word_str[i]), any_char);
 	break;
@@ -2190,8 +2315,8 @@ Transfer::applyWord(wstring const &word_str)
   ms.step(L'$');
 }
 
-void 
-Transfer::setPreBilingual(bool value) 
+void
+Transfer::setPreBilingual(bool value)
 {
   preBilingual = value;
 }
@@ -2202,8 +2327,8 @@ Transfer::getPreBilingual(void) const
   return preBilingual;
 }
 
-void 
-Transfer::setUseBilingual(bool value) 
+void
+Transfer::setUseBilingual(bool value)
 {
   useBilingual = value;
 }
